@@ -13,15 +13,18 @@
 #'@importFrom "ggplot2" "aes" "element_blank" "geom_histogram" "geom_line" "geom_ribbon" "ggplot" "labs" "scale_y_continuous"
 #'@importFrom "scales" "rescale"
 #'@importFrom "stats" "anova"
-#'
+#'@importFrom "cowplot" "draw_label"
+#'@import "cowplot"
 #'@return a picture
 
 #'@format NULL
 #'@usage NULL
 #'@export
-#'@examples library(rms)
+#'@examples
+#'library(rms)
 #'library(ggplot2)
 #'library(scales)
+#'library(cowplot)
 #'dt<-smoke
 #'dd<-datadist(dt)
 #'options(datadist='dd')
@@ -41,15 +44,17 @@ utils::globalVariables(c('theme_bw',
                          'after_stat',
                          'scale_fill_discrete',
                          'guides',
-                         'guides scale_fill_discrete'
+                         'guides scale_fill_discrete',
+                         'draw_label',
+                         'geom_hline'
 ))
 
 
 ggrcs<-function(data,fit,x,group=NULL,groupcol=NULL,histlimit=NULL,histbinwidth=NULL,histcol=NULL,
                 linetype=NULL,linesize=NULL,ribalpha=NULL,ribcol=NULL,xlab=NULL,ylab=NULL,
-                leftaxislimit=NULL,lift=TRUE,Pvalue=NULL,P.Nonlinear=FALSE,liftname=NULL,
-                title=NULL,xP.Nonlinear=NULL,yP.Nonlinear=NULL,two.group.label=NULL,
-                bordercol=NULL,twotag.name=NULL,...){
+                leftaxislimit=NULL,lift=TRUE,P.Nonlinear=TRUE,liftname=NULL,riblinecol=NULL,linecol=NULL,
+                title=NULL,px=NULL,py=NULL,two.group.label=NULL,histalpha=NULL,linealpha=NULL,
+                bordercol=NULL,twotag.name=NULL,dec=NULL,fontsize=12,fontfamily="serif",colset=NULL,...){
   density<-NULL;yhat<-NULL;lower<-NULL;upper<-NULL
   if (missing(data)) {stop("data is miss.")}
   if (missing(fit)) {stop("fit is miss.")}
@@ -60,12 +65,23 @@ ggrcs<-function(data,fit,x,group=NULL,groupcol=NULL,histlimit=NULL,histbinwidth=
   call <- match.call()
   data<-as.data.frame(data)
   fit <- fit;assign("fit", fit);
-  if (!missing(group)) {assign("group",group)}
+  if (is.null(dec)) {dec<-3} else {dec<-dec}
+  an<-anova(fit)
+  if (any(class(fit)=="cph"|class(fit)=="lrm")==T) {
+    p.value <- an[2, 3]
+    p.overall <- an[1, 3]
+  } else {
+    p.value <- an[2, 5]
+    p.overall <- an[1, 5]
+  }
+  p.value<-pvformat(p.value,dec)
+  p.overall<-pvformat(p.overall,dec)
+  if (!missing(group)) {
+    assign("group",group)
+    groupname<-levels(factor(data[,group]))
+  }
   if (!missing(group)) {
     #two group
-    an<-anova(fit)
-    P.value<-an[2,3]
-    P.value<-round(P.value,3)
     dt<-as.data.frame(data)
     x1<-x #get name
     x<-dt[,x]#predect data x
@@ -84,38 +100,61 @@ ggrcs<-function(data,fit,x,group=NULL,groupcol=NULL,histlimit=NULL,histbinwidth=
     #####ggplot set
     if (missing(histlimit)) {histlimit<-c(yminlower,ymaxupper)} else {assign("histlimit",histlimit)}
     if (missing(histbinwidth)) {histbinwidth<-0.8} else {assign("histbinwidth",histbinwidth)}
-    if (missing(histcol)) {histcol<-"green"} else {assign("histcol",histcol)}
+    if (missing(histcol)) {histcol<-"plum1"} else {assign("histcol",histcol)}
     if (missing(linetype)) {linetype<-1} else {assign("linetype",linetype)}
     if (missing(linesize)) {linesize<-1} else {assign("linesize",linesize)}
     if (missing(ribalpha)) {ribalpha<-0.3} else {assign("ribalpha",ribalpha)}
-    if (missing(ribcol)) {ribcol<-"red"} else {assign("ribcol",ribcol)}
-    if (missing(Pvalue)) {Pvalue<-P.value} else {assign("Pvalue",Pvalue)}
+    if (missing(ribcol)) {ribcol<-"lightblue3"} else {assign("ribcol",ribcol)}
     if (missing(xlab)) {xlab<-x1} else {assign("xlab",xlab)}
     if (missing(ylab)) {ylab<-"Outcome Prediction Incidence"} else {assign("ylab",ylab)}
+    if (missing(histalpha)) {histalpha<-0.5} else {assign("histalpha",histalpha)}
+    if (missing(linealpha)) {linealpha<-0.7} else {assign("linealpha",linealpha)}
+    if (missing(linecol)) {linecol<-ribcol} else {linecol<-linecol}
     if (missing(leftaxislimit)) {leftaxislimit<-c(dmin,dmax)} else {assign("leftaxislimit",leftaxislimit)}
     if (missing(title)) {title<-"The relationship between the variable and the predicted probability"} else {assign("title",title)}
     if (!missing(groupcol)) {assign("groupcol",groupcol)}
-    if (missing(twotag.name)) {twotag.name<-c("1","2")} else {assign("twotag.name",twotag.name)}
+    if (missing(twotag.name)) {twotag.name<-groupname} else {assign("twotag.name",twotag.name)}
     if (missing(bordercol)) {bordercol<-"black"} else {assign("bordercol",bordercol)}
     if (missing(groupcol)) {
-      P<-ggplot()+
+      groupcol=c("plum1","lightblue3")
+      if ((!missing(colset))) {
+        if (colset=="A")  {
+          groupcol<-c("darkseagreen1","gold1")
+        }
+        if (colset=="B")  {
+          groupcol<-c("darkseagreen1","burlywood3")
+        }
+        if (colset=="C")  {
+          groupcol<-c("cornsilk","palegreen")
+        }
+        if (colset=="D")  {
+          groupcol<-c("springgreen","tan3")
+        }
+      }
+      p<-ggplot()+
         geom_histogram(data=pre0,aes(x=x,y =rescale(after_stat(density),histlimit),fill=group,group=group),
-                       binwidth = histbinwidth,color=bordercol)+scale_fill_discrete(labels = twotag.name)+
-        geom_line(data=pre0,aes(x,yhat,colour=group,group=group),linetype=linetype,size=linesize,alpha = 0.9)+guides(colour = "none")+
+                       binwidth = histbinwidth,color=bordercol,alpha = histalpha)+
+        scale_fill_manual(values=groupcol,labels = twotag.name)+
+        geom_line(data=pre0,aes(x,yhat,colour=group,group=group),
+                  linetype=linetype,linewidth=linesize,alpha = linealpha)+
+        scale_colour_manual(values=groupcol)+
         geom_ribbon(data=pre0,aes(x=x,ymin=lower,ymax=upper,fill=group,group=group),alpha =ribalpha)+
+        guides(colour = "none")+
         theme_bw()+
         theme(panel.grid.major = element_blank(),panel.grid.minor = element_blank())+
         xlab(xlab)+
         ylab(ylab)+
         labs(title=title)
     } else {
-      P<-ggplot()+
+      p<-ggplot()+
         geom_histogram(data=pre0,aes(x=x,y =rescale(after_stat(density),histlimit),fill=group,group=group),
-                       binwidth = histbinwidth,color=bordercol)+
+                       binwidth = histbinwidth,color=bordercol,alpha = histalpha)+
         scale_fill_manual(values=groupcol,labels = twotag.name)+
-        geom_line(data=pre0,aes(x,yhat,colour=group,group=group),linetype=linetype,size=linesize,alpha = 0.9)+
+        geom_line(data=pre0,aes(x,yhat,colour=group,group=group),
+                  linetype=linetype,linewidth=linesize,alpha = linealpha)+
         scale_colour_manual(values=groupcol)+
-        geom_ribbon(data=pre0,aes(x=x,ymin=lower,ymax=upper,fill=group,group=group),alpha =ribalpha)+guides(colour = "none")+
+        geom_ribbon(data=pre0,aes(x=x,ymin=lower,ymax=upper,fill=group,group=group),alpha =ribalpha)+
+        guides(colour = "none")+
         theme_bw()+
         theme(panel.grid.major = element_blank(),panel.grid.minor = element_blank())+
         xlab(xlab)+
@@ -124,9 +163,6 @@ ggrcs<-function(data,fit,x,group=NULL,groupcol=NULL,histlimit=NULL,histbinwidth=
     }
   } else {
     #one group
-    an<-anova(fit)
-    P.value<-an[2,3]
-    P.value<-round(P.value,3)
     dt<-data
     x1<-x
     x<-dt[,x]
@@ -141,21 +177,52 @@ ggrcs<-function(data,fit,x,group=NULL,groupcol=NULL,histlimit=NULL,histbinwidth=
     #####ggplot set
     if (missing(histlimit)) {histlimit<-c(yminlower,ymaxupper)} else {assign("histlimit",histlimit)}
     if (missing(histbinwidth)) {histbinwidth<-0.8} else {assign("histbinwidth",histbinwidth)}
-    if (missing(histcol)) {histcol<-"green"} else {assign("histcol",histcol)}
+    if (missing(histcol)) {histcol<-"plum1"} else {assign("histcol",histcol)}
     if (missing(linetype)) {linetype<-1} else {assign("linetype",linetype)}
     if (missing(linesize)) {linesize<-1} else {assign("linesize",linesize)}
     if (missing(ribalpha)) {ribalpha<-0.3} else {assign("ribalpha",ribalpha)}
-    if (missing(ribcol)) {ribcol<-"red"} else {assign("ribcol",ribcol)}
-    if (missing(Pvalue)) {Pvalue<-P.value} else {assign("Pvalue",Pvalue)}
+    if (missing(ribcol)) {ribcol<-"lightblue3"} else {assign("ribcol",ribcol)}
     if (missing(xlab)) {xlab<-x1} else {assign("xlab",xlab)}
     if (missing(ylab)) {ylab<-"Outcome Prediction Incidence"} else {assign("ylab",ylab)}
+    if (missing(histalpha)) {histalpha<-0.5} else {assign("histalpha",histalpha)}
+    if (missing(linealpha)) {linealpha<-0.7} else {assign("linealpha",linealpha)}
+    if (missing(linecol)) {linecol<-ribcol} else {linecol<-linecol}
     if (missing(leftaxislimit)) {leftaxislimit<-c(dmin,dmax)} else {assign("leftaxislimit",leftaxislimit)}
     if (missing(title)) {title<-"The relationship between the variable and the predicted probability"} else {assign("title",title)}
-    P<-ggplot(pre0,aes(x=x))+
-      geom_histogram(aes(x=x,y =rescale(after_stat(density),histlimit)),binwidth = histbinwidth,fill=histcol,colour="black")+
-      geom_line(data=pre0,aes(x,yhat),linetype=linetype,size=linesize,alpha = 0.9,colour=ribcol)+
-      geom_ribbon(data=pre0,aes(ymin = lower, ymax = upper),alpha = ribalpha,fill=ribcol)+
-      theme_bw()+
+    if ((!missing(colset))) {
+      if (colset=="A")  {
+        histcol<-c("darkseagreen1")
+        ribcol<-c("gold1")
+      }
+      if (colset=="B")  {
+        histcol<-c("darkseagreen1")
+        ribcol<-c("burlywood3")
+      }
+      if (colset=="C")  {
+        histcol<-c("cornsilk")
+        ribcol<-c("palegreen")
+      }
+      if (colset=="D")  {
+        histcol<-c("springgreen")
+        ribcol<-c("tan3")
+      }
+    }
+    if (missing(riblinecol)) {
+      p<-ggplot(pre0,aes(x=x))+
+        geom_histogram(aes(x=x,y =rescale(after_stat(density),histlimit)),
+                       binwidth = histbinwidth,fill=histcol,colour="black",alpha=histalpha)+
+        geom_line(data=pre0,aes(x,yhat),linetype=linetype,linewidth=linesize,alpha = linealpha,colour=linecol)+
+        geom_ribbon(data=pre0,aes(ymin = lower, ymax = upper),alpha = ribalpha,fill=ribcol)
+    } else {
+      riblinecol<-riblinecol
+      p<-ggplot(pre0,aes(x=x))+
+        geom_histogram(aes(x=x,y =rescale(after_stat(density),histlimit)),
+                       binwidth = histbinwidth,fill=histcol,colour="black",alpha=histalpha)+
+        geom_line(data=pre0,aes(x,yhat),linetype=linetype,linewidth=linesize,alpha = linealpha,colour=ribcol)+
+        geom_ribbon(data=pre0,aes(ymin = lower, ymax = upper),
+                    alpha = ribalpha,fill=ribcol,col=riblinecol)
+    }
+    p<-p+theme_bw()+
       theme(panel.grid.major = element_blank(),panel.grid.minor = element_blank())+
       xlab(xlab)+
       ylab(ylab)+
@@ -164,18 +231,36 @@ ggrcs<-function(data,fit,x,group=NULL,groupcol=NULL,histlimit=NULL,histbinwidth=
   ###LIFT
   if (lift==TRUE) {
     if (missing(liftname)) {liftname<-"density"} else {assign("liftname",liftname)}
-    P<-P+scale_y_continuous(sec.axis = sec_axis( ~rescale(.,leftaxislimit),
+    p<-p+scale_y_continuous(sec.axis = sec_axis( ~rescale(.,leftaxislimit),
                                                  name = liftname))
   }
   if (P.Nonlinear==TRUE) {
+    if (any(grepl("<",p.value))) {
+      p.value <- paste0("P for nonlinear", p.value)
+    }
+    else {
+      p.value <- paste0("P for nonlinear", " = ",
+                        p.value)
+    }
+    if (any(grepl("<",p.overall))) {
+      p.overall <- paste0("P for overall", p.overall)
+    }
+    else {
+      p.overall <- paste0("P for overall", " = ",
+                          p.overall)
+    }
     text<- ""
-    text<- paste("P for Nonlinear:",Pvalue,sep="")
-    if (missing(xP.Nonlinear)) {xP.Nonlinear<-max(x)*0.3} else {assign("xP.Nonlinear",xP.Nonlinear)}
-    if (missing(yP.Nonlinear)) {yP.Nonlinear<-max(pre0$upper)*0.95} else {assign("yP.Nonlinear",yP.Nonlinear)}
-    #xP.Nonlinear<-max(x)*0.3
-    #yP.Nonlinear<-max(pre0$upper)*0.95
-    P<-P+annotate("text", x=xP.Nonlinear,y=yP.Nonlinear,label=text,size=5)
+    text<- paste(p.overall, p.value, sep = "\n")
+    if (missing(px)) {px<-max(x)*0.3} else {assign("px",px)}
+    if (missing(py)) {py<-max(pre0$upper)*0.95} else {assign("py",py)}
+    px<-round(px)
+    py<-round(py)
+    #px<-max(x)*0.3
+    #py<-max(pre0$upper)*0.95
+    p<-p+cowplot::draw_label(text, size = fontsize,
+                    fontfamily = fontfamily, x = px, y = py, hjust = 0,
+                    vjust = 1)
   }
-  P
+  p
 }
 
